# where are the Max/MSP SDK header files?
# for now, please use MaxSDK-5.?.?, avoid MaxSDK-6.?.?
# you can run your externals with Max 6 nevertheless

MAXSDKPATH=/Applications/MaxMSP\ 4.6/MaxMSP-SDK/c74support

###############################################################

# prefix for flext installation
# headers are in $(FLEXTPREFIX)/include/flext
# libraries are in $(FLEXTPREFIX)/lib
# build system is in $(FLEXTPREFIX)/lib/flext

FLEXTPREFIX=/usr/local

###############################################################

# where should the external be built?
OUTPATH=max-darwin

# where should the external be installed?
INSTPATH=/Applications/MaxMSP\ 4.6/Cycling\ \'74/externals/flext

# where should the initialization files be installed?
INITPATH=/Applications/MaxMSP\ 4.6/Cycling\ \'74/init

# where should the help files be installed?
HELPPATH=/Applications/MaxMSP\ 4.6/max-help/flext

###############################################################

# STK (synthesis tool kit) support
# http://ccrma.stanford.edu/software/stk

# where to find the STK header files (e.g. stk.h)
#STK_INC=/usr/local/include/stk

# where to find the STK library (normally libstk.a)
# (comment out STK_LIB if you don't use STK)
#STK_LIB=/usr/local/lib/libstk.a

###############################################################

# SndObj support
# http://music.nuim.ie//musictec/SndObj

# where to find the SndObj header files (e.g. sndobj.h)
#SNDOBJ_INC=/usr/local/include/sndobj

# where to find the SndObj library (normally libsndobj.a)
# (comment out STK_LIB if you don't use SndObj)
#SNDOBJ_LIB=/usr/local/lib/libsndobj.a

###############################################################

# make flags (e.g. use multiprocessor)
MFLAGS=-j 2

# user defined compiler flags
UFLAGS += -ffast-math -mmacosx-version-min=10.6 -march=native

# user defined linker flags
LDFLAGS += -mmacosx-version-min=10.6

# user defined optimization flags
# for now, don't use -O3 !(Max will hang)
OFLAGS += -Os -mtune=native # -ftree-vectorize

# user defined debugging flags
DFLAGS +=

# architecture-specific flags (optional)
UFLAGS_ppc += -faltivec 
OFLAGS_ppc += 
DFLAGS_ppc +=

UFLAGS_i386 +=
OFLAGS_i386 += 
DFLAGS_i386 +=

UFLAGS_x86_64 +=
OFLAGS_x86_64 += 
DFLAGS_x86_64 +=

# list of architectures to build
ARCH=i386 #ppc x86_64 ppc64

# SDK for 10.4
#OSXSDK=/Developer/SDKs/MacOSX10.4u.sdk
