# where are the Max/MSP SDK header files?
# you should have the latest version!
MAXSDKPATH="%ProgramFiles%\MaxMSP 4.5/maxmspsdk_win/4.5 headers/c74support"

###############################################################

# where do/should the flext headers reside/be built?
FLEXTINC="%CommonProgramFiles%/Cycling '74/flext"

# where do/should the flext static libraries reside/be built?
FLEXTLIB=$(FLEXTINC)

# where do/should the flext shared libraries reside/be built?
# (a good place is the MaxMSP program folder)
FLEXTSHLIB="%ProgramFiles%\MaxMSP 4.5"

###############################################################

# where should the external be built?
# (path for temporary files)
OUTPATH=max-mingw

# where should the external be installed?
INSTPATH="%CommonProgramFiles%/Cycling '74/externals/flext"

# where should the initialization files be installed?
INITPATH="%CommonProgramFiles%/Cycling '74/init"

# where should the help files be installed?
HELPPATH="%ProgramFiles%/MaxMSP 4.5/max-help/flext"

###############################################################

# where is the STK installation?
# (uncomment the line with # to disable STK support)
# --------------------------------------------------------------------
# In order to use STK you must have an Stk.obj file
# made with mingw
#   g++ -c -mno-cygwin -O3 -I ../include Stk.cpp -o Stk.obj
#
#STK=c:\data\prog\audio\stk

# where is the Sndobjs installation?
# (uncomment the line with # to disable SNDOBJ support)
# --------------------------------------------------------------------
# Please be sure to delete an eventual m_pd.h file from the SNDOBJ include folder!!
# The shared flext library links to the libsndobj.a library.
#
#SNDOBJ=c:\data\prog\audio\sndobj

###############################################################

# make flags (e.g. use multiprocessor)
#MFLAGS=-j 2

# user defined compiler flags
# (check if they match your system!)
UFLAGS=-msse -ffast-math

# user defined optimization flags
# (check if they match your system!)
OFLAGS=-O3
# optimizations for Pentium 4
OFLAGS+=-march=pentium4 
