# where is the PD installation?
# (this should point to the main folder, which has a "src" (PD Vanilla) or "include" (PD extended) subfolder)
PDPATH="%ProgramFiles%\pd"

# where is MS VC++?
# (not necessary if the build is run with the compiler environment)
# MSVCPATH="%ProgramFiles%\Microsoft Visual Studio .NET 2003\Vc7"

###############################################################

# version of the pthreads library (the one coming with PD has version number 1)
PTHREADSVERSION=1

# path to pthreads headers (PD comes with some in the src folder)
PTHREADSINC=$(PDPATH)\src

# path to pthreads link library (PD comes with one in the bin folder)
PTHREADSLIB=$(PDPATH)\bin

###############################################################

# path to lockfree library
LOCKFREEPATH=$(FLEXTINC)\source\lockfree

###############################################################

# where do/should the flext headers reside/be built?
FLEXTINC=$(PDPATH)\flext

# where do/should the flext static libraries reside/be built?
FLEXTLIB=$(FLEXTINC)

# where do/should the flext shared libraries reside/be built?
FLEXTSHLIB=$(PDPATH)\bin

###############################################################

# where should the external be built?
# (relative one-level subpath to project folder)
OUTPATH=pd-msvc

# where should externals be installed?
INSTPATH=$(PDPATH)\extra

###############################################################

# where is the STK installation?
# (uncomment the line with # to disable STK support)
# --------------------------------------------------------------------
# In order to use STK you must have an stk.lib library
# Using MSVC you can build it as following (from the STK main folder):
#   cl /c /GR /EHsc /Iinclude /MT /Ox src\*.cpp
#     (in case you define DYNAMIC=1 below, omit the /MT flag and set /MD instead)
#     (you will get errors for some files, you can ignore them in the flext context)
#   lib *.obj /out:stk.lib
#

# where to find the STK header files (e.g. stk.h)
#STK_INC=%HOMEDRIVE%\%HOMEPATH%\stk\include

# where to find the STK library (normally stk.lib)
# (comment out STK_LIB if you don't use STK)
#STK_LIB=%HOMEDRIVE%\%HOMEPATH%\stk\stk.lib

# where is the Sndobjs installation?
# (uncomment the line with # to disable SNDOBJ support)
# --------------------------------------------------------------------
# Please be sure to delete an eventual m_pd.h file from the SNDOBJ include folder!!
# The shared flext library links to the SndObj.lib file from the SNDOBJ distribution.
# Flext must be compiled with the same compiler version as the SNDOBJ lib (default MSVC6)
#

# where to find the SndObj header files (e.g. SndObj.h)
#SNDOBJ_INC=%HOMEDRIVE%\%HOMEPATH%\SndObj\include

# where to find the SndObj library (normally SndObj.lib)
# (comment out SNDOBJ_LIB if you don't use SndObj)
#SNDOBJ_LIB=%HOMEDRIVE%\%HOMEPATH%\SndObj\lib\SndObj.lib

###############################################################

# user defined compiler flags
# (check if they match your system!)
UFLAGS=

# user defined optimization flags
# (check if they match your system!)
OFLAGS=/Ox
# optimizations for Pentium 4
OFLAGS=$(OFLAGS) /G6 /arch:SSE

# comment if you want to compile for Windows 95/98/ME
WINNT=1

# uncomment to link against dynamic C runtime libraries
# (don't use this if you want to distribute the built product)
#DYNAMIC=1
